using System;
using System.IO;
using GTA;
using GTA.Native;

public class IV_CE_OvertakeFix : Script
{
    private const float TRUCK_DOT_PRODUCT_THRESHOLD = -0.96f;
    private const float CAR_DOT_PRODUCT_THRESHOLD = -0.94f;
    private const float CAR_DETECTION_DISTANCE = 6.7f;
    private const float TRUCK_DETECTION_DISTANCE = 9.0f;

    private const int FREEZE_DURATION = 16500;

    private readonly string logFilePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "IV_CE_OvertakeFix.log");

    private Vehicle lastFrozenTruck;
    private DateTime lastTruckFrozenTime;
    private bool playerMovedSinceTruckUnfreeze = true;

    private Vehicle lastStoppedCar;
    private DateTime lastCarStoppedTime;
    private bool playerMovedSinceCarUnfreeze = true;

    private readonly uint[] truckHashes = 
    {
        1917016601u, 1938952078u, 2053223216u, 850991848u,
        1353720154u, 904750859u, 569305213u, 2157618379u,
        1677715180u, 3195007082u, 3581397346u
    };

    private bool loggingEnabled = false;

    public IV_CE_OvertakeFix()
    {
        Interval = 100;
        Tick += IV_CE_OvertakeFix_Tick;
        InitializeLogFile();
    }

    private void InitializeLogFile()
    {
        if (loggingEnabled && !File.Exists(logFilePath))
        {
            File.WriteAllText(logFilePath, "IV_CE_OvertakeFix Log File\n\n");
        }
    }

    private void LogMessage(string message)
    {
        if (loggingEnabled)
        {
            File.AppendAllText(logFilePath, DateTime.Now + ": " + message + Environment.NewLine);
        }
    }

    private void IV_CE_OvertakeFix_Tick(object sender, EventArgs e)
    {
        try
        {
            if (!Player.Character.isInVehicle()) return;

            Vehicle playerVeh = Player.Character.CurrentVehicle;
            if (playerVeh == null || !playerVeh.Exists()) return;

            bool playerStopped = playerVeh.Speed < 0.1f;
            if (!playerStopped)
            {
                playerMovedSinceTruckUnfreeze = true;
                playerMovedSinceCarUnfreeze = true;
            }
            int recognitionDelay = playerStopped ? 60000 : 1;

            Vehicle vehBehind = FindVehicleBehind(playerVeh, recognitionDelay);
            if (vehBehind != null)
            {
                LogAndStopVehicle(vehBehind, playerStopped);
            }
            else
            {
                UnfreezeVehiclesIfNeeded(playerStopped);
            }

            MaintainCarStop(playerStopped);
        }
        catch (Exception ex)
        {
            LogMessage("Error in IV_CE_OvertakeFix: " + ex.Message);
        }
    }

    private Vehicle FindVehicleBehind(Vehicle playerVeh, int recognitionDelay)
    {
        Vehicle vehBehind = null;
        float minDistance = float.MaxValue;
        Vector3 playerPosition = playerVeh.Position;
        Vector3 playerDirection = playerVeh.Direction;

        foreach (Vehicle veh in World.GetAllVehicles())
        {
            if (IsValidVehicle(veh, playerVeh, recognitionDelay))
            {
                float distance = (veh.Position - playerPosition).Length();
                if (distance < minDistance && IsInDetectionRange(veh, playerVeh, distance, playerPosition, playerDirection))
                {
                    vehBehind = veh;
                    minDistance = distance;
                }
            }
        }

        return vehBehind;
    }

    private bool IsValidVehicle(Vehicle veh, Vehicle playerVeh, int recognitionDelay)
    {
        if (veh == playerVeh || !veh.Exists()) return false;

        if (IsTruck(veh.Model.Hash))
        {
            if (veh == lastFrozenTruck && (DateTime.Now - lastTruckFrozenTime).TotalMilliseconds < recognitionDelay)
                return false;
            if (!playerMovedSinceTruckUnfreeze) return false;
        }
        else
        {
            if (veh == lastStoppedCar && (DateTime.Now - lastCarStoppedTime).TotalMilliseconds < recognitionDelay)
                return false;
            if (!playerMovedSinceCarUnfreeze) return false;
        }
        return true;
    }

    private bool IsInDetectionRange(Vehicle veh, Vehicle playerVeh, float distance, Vector3 playerPosition, Vector3 playerDirection)
    {
        float maxDetectionDistance = IsTruck(veh.Model.Hash) ? TRUCK_DETECTION_DISTANCE : CAR_DETECTION_DISTANCE;
        return distance <= maxDetectionDistance && IsInDirection(playerVeh, veh, playerPosition, playerDirection);
    }

    private bool IsInDirection(Vehicle playerVeh, Vehicle veh, Vector3 playerPosition, Vector3 playerDirection)
    {
        Vector3 toVehicle = veh.Position - playerPosition;
        float toVehicleMagnitudeSquared = toVehicle.X * toVehicle.X + toVehicle.Y * toVehicle.Y;
        if (toVehicleMagnitudeSquared < 0.0001f) return false;

        float dotProduct = (playerVeh.Direction.X * toVehicle.X + playerVeh.Direction.Y * toVehicle.Y) / (float)Math.Sqrt(toVehicleMagnitudeSquared);
        float threshold = IsTruck(veh.Model.Hash) ? TRUCK_DOT_PRODUCT_THRESHOLD : CAR_DOT_PRODUCT_THRESHOLD;
        return dotProduct <= threshold;
    }

    private void LogAndStopVehicle(Vehicle vehBehind, bool playerStopped)
    {
        //Game.DisplayText("Detected vehicle behind (Hash): " + vehBehind.Model.Hash);
        LogMessage("Detected vehicle behind (Hash): " + vehBehind.Model.Hash);

        Ped driver = vehBehind.GetPedOnSeat(VehicleSeat.Driver);
        if (driver != null && driver.Exists() && driver != Player.Character && playerStopped)
        {
            driver.Task.ClearAll();

            if (IsTruck(vehBehind.Model.Hash))
            {
                vehBehind.Speed = 0;
                vehBehind.FreezePosition = true;
                lastFrozenTruck = vehBehind;
                lastTruckFrozenTime = DateTime.Now;
                LogMessage("Stopped truck (Hash): " + vehBehind.Model.Hash);
            }
            else
            {
                Function.Call("SET_VEHICLE_FORWARD_SPEED", vehBehind, 0);
                driver.Task.StandStill(100);
                if (lastStoppedCar == null || !lastStoppedCar.Exists() || lastStoppedCar.Model.Hash != vehBehind.Model.Hash)
                {
                    lastStoppedCar = vehBehind;
                    lastCarStoppedTime = DateTime.Now;
                }
                LogMessage("Stopped car (Hash): " + vehBehind.Model.Hash);
            }
        }
    }

    private void MaintainCarStop(bool playerStopped)
    {
        if (lastStoppedCar != null && lastStoppedCar.Exists() && playerStopped)
        {
            double carStoppedDuration = (DateTime.Now - lastCarStoppedTime).TotalMilliseconds;
            if (carStoppedDuration < FREEZE_DURATION)
            {
                Ped driver = lastStoppedCar.GetPedOnSeat(VehicleSeat.Driver);
                if (driver != null && driver.Exists() && driver != Player.Character)
                {
                    driver.Task.ClearAll();
                    Function.Call("SET_VEHICLE_FORWARD_SPEED", lastStoppedCar, 0);
                    driver.Task.StandStill(100);
                }
            }
        }
    }

    private void UnfreezeVehiclesIfNeeded(bool playerStopped)
    {
        if (lastFrozenTruck != null && lastFrozenTruck.Exists())
        {
            double truckFrozenDuration = (DateTime.Now - lastTruckFrozenTime).TotalMilliseconds;
            //Game.DisplayText("Truck Frozen time elapsed: " + truckFrozenDuration + " ms");
            if (!playerStopped || truckFrozenDuration >= FREEZE_DURATION)
            {
                int hash = lastFrozenTruck.Model.Hash;
                lastFrozenTruck.FreezePosition = false;
                lastFrozenTruck.Speed = 0;
                lastFrozenTruck = null;
                playerMovedSinceTruckUnfreeze = false;
                //Game.DisplayText("Unfreezing truck (Hash): " + hash);
                LogMessage("Unfreezing truck (Hash): " + hash);
            }
        }
        else
        {
            lastFrozenTruck = null;
        }

        if (lastStoppedCar != null && lastStoppedCar.Exists())
        {
            double carStoppedDuration = (DateTime.Now - lastCarStoppedTime).TotalMilliseconds;
            //Game.DisplayText("Car Stopped time elapsed: " + carStoppedDuration + " ms");
            if (!playerStopped || carStoppedDuration >= FREEZE_DURATION)
            {
                int hash = lastStoppedCar.Model.Hash;
                Ped driver = lastStoppedCar.GetPedOnSeat(VehicleSeat.Driver);
                if (driver != null && driver.Exists())
                {
                    driver.Task.ClearAll();
                }
                lastStoppedCar = null;
                playerMovedSinceCarUnfreeze = false;
                //Game.DisplayText("Unfreezing car (Hash): " + hash);
                LogMessage("Unfreezing car (Hash): " + hash);
            }
        }
        else
        {
            lastStoppedCar = null;
        }
    }

    private bool IsTruck(int modelHash)
    {
        foreach (uint hash in truckHashes)
        {
            if (hash == modelHash)
                return true;
        }
        return false;
    }
}
    
    
    
    
    
    
                
            
                
                
                
    
        
        
                
